print("import: 'eventlet'")
import eventlet

print("import: 'eventlet.green'")
import eventlet.green

print("import: 'eventlet.green.OpenSSL'")
import eventlet.green.OpenSSL

print("import: 'eventlet.green.http'")
import eventlet.green.http

print("import: 'eventlet.green.urllib'")
import eventlet.green.urllib

print("import: 'eventlet.greenio'")
import eventlet.greenio

print("import: 'eventlet.hubs'")
import eventlet.hubs

print("import: 'eventlet.support'")
import eventlet.support

